@@ -209,8 +209,18 @@ int lws_open(const char *__file, int __oflag, ...)
 		|| ((__oflag & O_TMPFILE) == O_TMPFILE)
 #endif
 	)
+#if defined(WIN32)
 		/* last arg is really a mode_t.  But windows... */
 		n = open(__file, __oflag, va_arg(ap, uint32_t));
+#else
+		/* ... and some other toolchains...
+		 *
+		 * error: second argument to 'va_arg' is of promotable type 'mode_t'
+		 * (aka 'unsigned short'); this va_arg has undefined behavior because
+		 * arguments will be promoted to 'int'
+		 */
+		n = open(__file, __oflag, (mode_t)va_arg(ap, unsigned int));
+#endif
 	else
 		n = open(__file, __oflag);
 	va_end(ap);
@@ -234,6 +244,10 @@ lws_pthread_self_to_tsi(struct lws_context *context)
 	struct lws_context_per_thread *pt = &context->pt[0];
 	int n;
 
+	/* case that we have SMP build, but don't use it */
+	if (context->count_threads == 1)
+		return 0;
+
 	for (n = 0; n < context->count_threads; n++) {
 		if (pthread_equal(ps, pt->self))
 			return n;
@@ -1165,9 +1179,11 @@ drain:
 }
 
 int
-lws_strcmp_wildcard(const char *wildcard, size_t len, const char *check)
+lws_strcmp_wildcard(const char *wildcard, size_t wlen, const char *check,
+		    size_t clen)
 {
-	const char *match[3], *wc[3], *wc_end = wildcard + len;
+	const char *match[3], *wc[3], *wc_end = wildcard + wlen,
+		   *cend = check + clen;
 	int sp = 0;
 
 	do {
@@ -1236,9 +1252,13 @@ lws_strcmp_wildcard(const char *wildcard, size_t len, const char *check)
 
 		/* we're looking for a post-* match... keep looking... */
 
-	} while (*check);
+	} while (check < cend);
 
-	return !!*wildcard;
+	/*
+	 * We reached the end of check, if also at end of wildcard we're OK
+	 */
+
+	return wildcard != wc_end;
 }
 
 #if LWS_MAX_SMP > 1
