@@ -116,7 +116,7 @@ lws_glib_dispatch(GSource *src, GSourceFunc x, gpointer userData)
 	GIOCondition cond;
 
 	cond = g_source_query_unix_fd(src, sub->tag);
-	eventfd.revents = cond;
+	eventfd.revents = (short)cond;
 
 	/* translate from glib event namespace to platform */
 
@@ -132,8 +132,8 @@ lws_glib_dispatch(GSource *src, GSourceFunc x, gpointer userData)
 	eventfd.events = eventfd.revents;
 	eventfd.fd = sub->wsi->desc.sockfd;
 
-	lwsl_debug("%s: wsi %p: fd %d, events %d\n", __func__, sub->wsi,
-			eventfd.fd, eventfd.revents);
+	lwsl_wsi_debug(sub->wsi, "fd %d, events %d",
+				 eventfd.fd, eventfd.revents);
 
 	pt = &sub->wsi->a.context->pt[(int)sub->wsi->tsi];
 	if (pt->is_destroyed)
@@ -177,7 +177,7 @@ lws_glib_hrtimer_cb(void *p)
 	us = __lws_sul_service_ripe(pt->pt_sul_owner, LWS_COUNT_PT_SUL_OWNERS,
 				    lws_now_usecs());
 	if (us) {
-		ms = us / LWS_US_PER_MS;
+		ms = (unsigned int)(us / LWS_US_PER_MS);
 		if (!ms)
 			ms = 1;
 
@@ -293,12 +293,21 @@ elops_accept_glib(struct lws *wsi)
 	return 0;
 }
 
+static int
+elops_listen_init_glib(struct lws_dll2 *d, void *user)
+{
+	struct lws *wsi = lws_container_of(d, struct lws, listen_list);
+
+	elops_accept_glib(wsi);
+
+	return 0;
+}
+
 static int
 elops_init_pt_glib(struct lws_context *context, void *_loop, int tsi)
 {
 	struct lws_context_per_thread *pt = &context->pt[tsi];
 	struct lws_pt_eventlibs_glib *ptpr = pt_to_priv_glib(pt);
-	struct lws_vhost *vh = context->vhost_list;
 	GMainLoop *loop = (GMainLoop *)_loop;
 
 	if (!loop)
@@ -307,24 +316,14 @@ elops_init_pt_glib(struct lws_context *context, void *_loop, int tsi)
 		context->pt[tsi].event_loop_foreign = 1;
 
 	if (!loop) {
-		lwsl_err("%s: creating glib loop failed\n", __func__);
+		lwsl_cx_err(context, "creating glib loop failed");
 
 		return -1;
 	}
 
 	ptpr->loop = loop;
 
-	/*
-	* Initialize all events with the listening sockets
-	* and register a callback for read operations
-	*/
-
-	while (vh) {
-		if (vh->lserv_wsi)
-			elops_accept_glib(vh->lserv_wsi);
-
-		vh = vh->vhost_next;
-	}
+	lws_vhost_foreach_listen_wsi(context, NULL, elops_listen_init_glib);
 
 	lws_glib_set_idle(pt);
 
@@ -344,7 +343,7 @@ elops_init_pt_glib(struct lws_context *context, void *_loop, int tsi)
  */
 
 static void
-elops_io_glib(struct lws *wsi, int flags)
+elops_io_glib(struct lws *wsi, unsigned int flags)
 {
 	struct lws_context_per_thread *pt = &wsi->a.context->pt[(int)wsi->tsi];
 	struct lws_wsi_eventlibs_glib *wsipr = wsi_to_priv_glib(wsi);
@@ -364,22 +363,22 @@ elops_io_glib(struct lws *wsi, int flags)
 
 	if (flags & LWS_EV_READ) {
 		if (flags & LWS_EV_STOP)
-			cond &= ~(G_IO_IN | G_IO_HUP);
+			cond &= (unsigned int)~(G_IO_IN | G_IO_HUP);
 		else
 			cond |= G_IO_IN | G_IO_HUP;
 	}
 
 	if (flags & LWS_EV_WRITE) {
 		if (flags & LWS_EV_STOP)
-			cond &= ~G_IO_OUT;
+			cond &= (unsigned int)~G_IO_OUT;
 		else
 			cond |= G_IO_OUT;
 	}
 
-	wsipr->w_read.actual_events = cond;
+	wsipr->w_read.actual_events = (uint8_t)cond;
 
-	lwsl_debug("%s: wsi %p, fd %d, 0x%x/0x%x\n", __func__, wsi,
-			wsi->desc.sockfd, flags, (int)cond);
+	lwsl_wsi_debug(wsi, "fd %d, 0x%x/0x%x", wsi->desc.sockfd,
+						flags, (int)cond);
 
 	g_source_modify_unix_fd(wsi_to_gsource(wsi), wsi_to_subclass(wsi)->tag,
 				cond);
@@ -420,25 +419,26 @@ elops_destroy_wsi_glib(struct lws *wsi)
 	wsi_to_subclass(wsi) = NULL;
 }
 
+static int
+elops_listen_destroy_glib(struct lws_dll2 *d, void *user)
+{
+	struct lws *wsi = lws_container_of(d, struct lws, listen_list);
+
+	elops_destroy_wsi_glib(wsi);
+
+	return 0;
+}
+
 static void
 elops_destroy_pt_glib(struct lws_context *context, int tsi)
 {
 	struct lws_context_per_thread *pt = &context->pt[tsi];
 	struct lws_pt_eventlibs_glib *ptpr = pt_to_priv_glib(pt);
-	struct lws_vhost *vh = context->vhost_list;
 
 	if (!pt_to_loop(pt))
 		return;
 
-	/*
-	 * Free all events with the listening sockets
-	 */
-	while (vh) {
-		if (vh->lserv_wsi)
-			elops_destroy_wsi_glib(vh->lserv_wsi);
-
-		vh = vh->vhost_next;
-	}
+	lws_vhost_foreach_listen_wsi(context, NULL, elops_listen_destroy_glib);
 
 	lws_gs_destroy(ptpr->idle);
 	lws_gs_destroy(ptpr->hrtimer);
@@ -490,6 +490,7 @@ static const struct lws_event_loop_ops event_loop_ops_glib = {
 	/* run_pt */			elops_run_pt_glib,
 	/* destroy_pt */		elops_destroy_pt_glib,
 	/* destroy wsi */		elops_destroy_wsi_glib,
+	/* foreign_thread */		NULL,
 
 	/* flags */			LELOF_DESTROY_FINAL,
 
@@ -506,6 +507,7 @@ const lws_plugin_evlib_t evlib_glib = {
 	.hdr = {
 		"glib event loop",
 		"lws_evlib_plugin",
+		LWS_BUILD_HASH,
 		LWS_PLUGIN_API_MAGIC
 	},
 
