@@ -30,8 +30,9 @@ lws_plat_asyncdns_init(struct lws_context *context, lws_sockaddr46 *sa46)
 	lws_async_dns_server_check_t s = LADNS_CONF_SERVER_CHANGED;
 	lws_sockaddr46 sa46t;
 	lws_tokenize_t ts;
-	int fd, n, ns = 0;
 	char ads[48], *r;
+	int fd, ns = 0;
+	ssize_t n;
 
 	r = (char *)context->pt[0].serv_buf;
 
@@ -53,7 +54,7 @@ lws_plat_asyncdns_init(struct lws_context *context, lws_sockaddr46 *sa46)
 				  LWS_TOKENIZE_F_MINUS_NONTERM |
 				  LWS_TOKENIZE_F_HASH_COMMENT);
 	do {
-		ts.e = lws_tokenize(&ts);
+		ts.e = (int8_t)lws_tokenize(&ts);
 		if (ts.e != LWS_TOKZE_TOKEN) {
 			ns = 0;
 			continue;
@@ -88,24 +89,3 @@ lws_plat_asyncdns_init(struct lws_context *context, lws_sockaddr46 *sa46)
 
 	return LADNS_CONF_SERVER_UNKNOWN;
 }
-
-/*
- * Platform-specific ntpclient server configuration
- */
-
-int
-lws_plat_ntpclient_config(struct lws_context *context)
-{
-#if defined(LWS_HAVE_GETENV)
-	char *ntpsrv = getenv("LWS_NTP_SERVER");
-
-	if (ntpsrv && strlen(ntpsrv) < 64) {
-		lws_system_blob_direct_set(lws_system_get_blob(context,
-					    LWS_SYSBLOB_TYPE_NTP_SERVER, 0),
-					    (const uint8_t *)ntpsrv,
-					    strlen(ntpsrv));
-		return 1;
-	}
-#endif
-	return 0;
-}
