@@ -28,6 +28,9 @@ static int
 secstream_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 	     void *in, size_t len)
 {
+#if defined(LWS_WITH_SERVER)
+	struct lws_context_per_thread *pt = &wsi->a.context->pt[(int)wsi->tsi];
+#endif
 	lws_ss_handle_t *h = (lws_ss_handle_t *)lws_get_opaque_user_data(wsi);
 	uint8_t buf[LWS_PRE + 1400];
 	lws_ss_state_return_t r;
@@ -42,14 +45,19 @@ secstream_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			 in ? (char *)in : "(null)");
 		if (!h)
 			break;
+
+#if defined(LWS_WITH_CONMON)
+		lws_conmon_ss_json(h);
+#endif
+
 		r = lws_ss_event_helper(h, LWSSSCS_UNREACHABLE);
 		if (r == LWSSSSRET_DESTROY_ME)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		h->wsi = NULL;
 		r = lws_ss_backoff(h);
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 		break;
 
 	case LWS_CALLBACK_CLOSED: /* server */
@@ -57,26 +65,49 @@ secstream_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		if (!h)
 			break;
 		lws_sul_cancel(&h->sul_timeout);
+
+#if defined(LWS_WITH_CONMON)
+		lws_conmon_ss_json(h);
+#endif
+
 		r = lws_ss_event_helper(h, LWSSSCS_DISCONNECTED);
 		if (r == LWSSSSRET_DESTROY_ME)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		if (h->wsi)
 			lws_set_opaque_user_data(h->wsi, NULL);
 		h->wsi = NULL;
 
+#if defined(LWS_WITH_SERVER)
+		lws_pt_lock(pt, __func__);
+		lws_dll2_remove(&h->cli_list);
+		lws_pt_unlock(pt);
+#endif
+
 		if (reason == LWS_CALLBACK_CLIENT_CLOSED) {
 			if (h->policy &&
 			    !(h->policy->flags & LWSSSPOLF_OPPORTUNISTIC) &&
 #if defined(LWS_WITH_SERVER)
 			    !(h->info.flags & LWSSSINFLAGS_ACCEPTED) && /* not server */
 #endif
-			    !h->txn_ok && !wsi->a.context->being_destroyed) {
+			    !wsi->a.context->being_destroyed) {
 				r = lws_ss_backoff(h);
 				if (r != LWSSSSRET_OK)
-					return _lws_ss_handle_state_ret(r, wsi, &h);
+					return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 				break;
 			}
+
+#if defined(LWS_WITH_SERVER)
+			if (h->info.flags & LWSSSINFLAGS_ACCEPTED) {
+				/*
+				 * was an accepted client connection to
+				 * our server, so the stream is over now
+				 */
+				lws_ss_destroy(&h);
+				return 0;
+			}
+#endif
+
 		}
 		break;
 
@@ -85,9 +116,15 @@ secstream_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		h->retry = 0;
 		h->seqstate = SSSEQ_CONNECTED;
 		lws_sul_cancel(&h->sul);
+#if defined(LWS_WITH_SYS_METRICS)
+		/*
+		 * If any hanging caliper measurement, dump it, and free any tags
+		 */
+		lws_metrics_caliper_report_hist(h->cal_txn, (struct lws *)NULL);
+#endif
 		r = lws_ss_event_helper(h, LWSSSCS_CONNECTED);
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 		break;
 
 	case LWS_CALLBACK_RECEIVE:
@@ -105,13 +142,13 @@ secstream_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 
 		r = h->info.rx(ss_to_userobj(h), (const uint8_t *)in, len, f);
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		return 0; /* don't passthru */
 
 	case LWS_CALLBACK_SERVER_WRITEABLE:
 	case LWS_CALLBACK_CLIENT_WRITEABLE:
-		// lwsl_notice("%s: ss %p: WRITEABLE\n", __func__, h);
+		// lwsl_notice("%s: %s: WRITEABLE\n", __func__, lws_ss_tag(h));
 		if (!h || !h->info.tx)
 			return 0;
 
@@ -126,14 +163,14 @@ secstream_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		if (r == LWSSSSRET_TX_DONT_SEND)
 			return 0;
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		f1 = lws_write_ws_flags(h->policy->u.http.u.ws.binary ?
 					   LWS_WRITE_BINARY : LWS_WRITE_TEXT,
 					!!(f & LWSSS_FLAG_SOM),
 					!!(f & LWSSS_FLAG_EOM));
 
-		n = lws_write(wsi, buf + LWS_PRE, buflen, f1);
+		n = lws_write(wsi, buf + LWS_PRE, buflen, (enum lws_write_protocol)f1);
 		if (n < (int)buflen) {
 			lwsl_info("%s: write failed %d %d\n", __func__,
 					n, (int)buflen);
@@ -153,8 +190,7 @@ secstream_ws(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 const struct lws_protocols protocol_secstream_ws = {
 	"lws-secstream-ws",
 	secstream_ws,
-	0,
-	0,
+	0, 0, 0, NULL, 0
 };
 /*
  * Munge connect info according to protocol-specific considerations... this
@@ -176,8 +212,6 @@ secstream_connect_munge_ws(lws_ss_handle_t *h, char *buf, size_t len,
 	size_t used_in, used_out;
 	lws_strexp_t exp;
 
-	lwsl_notice("%s\n", __func__);
-
 	/* i.path on entry is used to override the policy urlpath if not "" */
 
 	if (i->path[0])
@@ -186,6 +220,12 @@ secstream_connect_munge_ws(lws_ss_handle_t *h, char *buf, size_t len,
 	if (!pbasis)
 		return 0;
 
+	if (h->policy->flags & LWSSSPOLF_HTTP_CACHE_COOKIES)
+		i->ssl_connection |= LCCSCF_CACHE_COOKIES;
+
+	if (h->policy->flags & LWSSSPOLF_PRIORITIZE_READS)
+		i->ssl_connection |= LCCSCF_PRIORITIZE_READS;
+
 	/* protocol aux is the path part ; ws subprotocol name */
 
 	i->path = buf;
@@ -199,11 +239,11 @@ secstream_connect_munge_ws(lws_ss_handle_t *h, char *buf, size_t len,
 
 	i->protocol = h->policy->u.http.u.ws.subprotocol;
 
-	lwsl_notice("%s: url %s, ws subprotocol %s\n", __func__, buf, i->protocol);
+	lwsl_ss_info(h, "url %s, ws subprotocol %s", buf, i->protocol);
 
 	return 0;
 }
 
 const struct ss_pcols ss_pcol_ws = {
-	"ws",  "http/1.1",  &protocol_secstream_ws, secstream_connect_munge_ws
+	"ws",  "http/1.1",  &protocol_secstream_ws, secstream_connect_munge_ws, 0, 0
 };
