@@ -84,7 +84,7 @@ lws_sspc_event_helper(lws_sspc_handle_t *h, lws_ss_constate_t cs,
 		return LWSSSSRET_OK;
 
 	h->h_in_svc = h;
-	ret = h->ssi.state((void *)((uint8_t *)&h[1]), NULL, cs, flags);
+	ret = h->ssi.state((void *)((uint8_t *)(h + 1)), NULL, cs, flags);
 	h->h_in_svc = NULL;
 
 	return ret;
@@ -201,7 +201,7 @@ callback_sspc_client(struct lws *wsi, enum lws_callback_reasons reason,
 {
 	lws_sspc_handle_t *h = (lws_sspc_handle_t *)lws_get_opaque_user_data(wsi);
 	size_t pktsize = wsi->a.context->max_http_header_data;
-	void *m = (void *)((uint8_t *)&h[1]);
+	void *m = (void *)((uint8_t *)(h + 1));
 	uint8_t *pkt = NULL, *p = NULL, *end = NULL;
 	lws_ss_state_return_t r;
 	uint64_t interval;
@@ -661,7 +661,7 @@ lws_sspc_create(struct lws_context *context, int tsi, const lws_ss_info_t *ssi,
 			ssi->streamtype);
 
 	memcpy(&h->ssi, ssi, sizeof(*ssi));
-	ua = (uint8_t *)&h[1];
+	ua = (uint8_t *)(h + 1);
 	memset(ua, 0, ssi->user_alloc);
 	p = (char *)ua + ssi->user_alloc;
 	memcpy(p, ssi->streamtype, strlen(ssi->streamtype) + 1);
@@ -1070,7 +1070,7 @@ lws_sspc_cancel_timeout(struct lws_sspc_handle *h)
 void *
 lws_sspc_to_user_object(struct lws_sspc_handle *h)
 {
-	return (void *)&h[1];
+	return (void *)(h + 1);
 }
 
 void
