@@ -1,7 +1,7 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2019 - 2020 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2019 - 2021 Andy Green <andy@warmcat.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -51,23 +51,6 @@
 
 #include <private-lib-core.h>
 
-/*
- * Because both sides of the connection share the conn, we allocate it
- * during accepted adoption, and both sides point to it.
- *
- * The last one of the accepted side and the onward side to close frees it.
- */
-
-struct conn {
-	struct lws_ss_serialization_parser parser;
-
-	lws_dsh_t		*dsh;	/* unified buffer for both sides */
-	struct lws		*wsi;	/* the client side */
-	lws_ss_handle_t		*ss;	/* the onward, ss side */
-
-	lws_ss_conn_states_t	state;
-};
-
 struct raw_pss {
 	struct conn		*conn;
 };
@@ -81,8 +64,54 @@ typedef struct ss_proxy_onward {
 	struct conn		*conn;
 } ss_proxy_t;
 
+void
+lws_proxy_clean_conn_ss(struct lws *wsi)
+{
+#if 0
+	lws_ss_handle_t *h = (lws_ss_handle_t *)wsi->a.opaque_user_data;
+	struct conn *conn = h->conn_if_sspc_onw;
 
-/* secure streams payload interface */
+	if (!wsi)
+		return;
+
+	if (conn && conn->ss)
+		conn->ss->wsi = NULL;
+#endif
+}
+
+
+void
+ss_proxy_onward_link_req_writeable(lws_ss_handle_t *h_onward)
+{
+	ss_proxy_t *m = (ss_proxy_t *)&h_onward[1];
+
+	if (m->conn->wsi) /* if possible, request client conn write */
+		lws_callback_on_writable(m->conn->wsi);
+}
+
+int
+__lws_ss_proxy_bind_ss_to_conn_wsi(void *parconn, size_t dsh_size)
+{
+	struct conn *conn = (struct conn *)parconn;
+	struct lws_context_per_thread *pt;
+
+	if (!conn || !conn->wsi || !conn->ss)
+		return -1;
+
+	pt = &conn->wsi->a.context->pt[(int)conn->wsi->tsi];
+
+	if (lws_fi(&conn->ss->fic, "ssproxy_dsh_create_oom"))
+		return -1;
+	conn->dsh = lws_dsh_create(&pt->ss_dsh_owner, dsh_size, 2);
+	if (!conn->dsh)
+		return -1;
+
+	__lws_lc_tag_append(&conn->wsi->lc, lws_ss_tag(conn->ss));
+
+	return 0;
+}
+
+/* Onward secure streams payload interface */
 
 static lws_ss_state_return_t
 ss_proxy_onward_rx(void *userobj, const uint8_t *buf, size_t len, int flags)
@@ -102,14 +131,49 @@ ss_proxy_onward_rx(void *userobj, const uint8_t *buf, size_t len, int flags)
 		flags |= LWSSS_FLAG_RIDESHARE;
 	}
 
-	n = lws_ss_serialize_rx_payload(m->conn->dsh, buf, len, flags, rsp);
+	/*
+	 * Apply SSS framing around this chunk of RX and stash it in the dsh
+	 * in ss -> proxy [ -> client] direction.  This can fail...
+	 */
+
+	if (lws_fi(&m->ss->fic, "ssproxy_dsh_rx_queue_oom"))
+		n = 1;
+	else
+		n = lws_ss_serialize_rx_payload(m->conn->dsh, buf, len,
+						flags, rsp);
 	if (n)
-		return n;
+		/*
+		 * We couldn't buffer this rx, eg due to OOM, let's escalate it
+		 * to be a "loss of connection", which it basically is...
+		 */
+		return LWSSSSRET_DISCONNECT_ME;
+
+	/*
+	 * Manage rx flow on the SS (onward) side according to our situation
+	 * in the dsh holding proxy->client serialized forwarding rx
+	 */
+
+	if (!m->conn->onward_in_flow_control && m->ss->wsi &&
+	    m->ss->policy->proxy_buflen_rxflow_on_above &&
+	    lws_dsh_get_size(m->conn->dsh, KIND_SS_TO_P) >=
+				m->ss->policy->proxy_buflen_rxflow_on_above) {
+		lwsl_info("%s: %s: rxflow disabling rx (%lu / %lu, hwm %lu)\n", __func__,
+				lws_wsi_tag(m->ss->wsi),
+				(unsigned long)lws_dsh_get_size(m->conn->dsh, KIND_SS_TO_P),
+				(unsigned long)m->ss->policy->proxy_buflen,
+				(unsigned long)m->ss->policy->proxy_buflen_rxflow_on_above);
+		/*
+		 * stop taking in rx once the onward wsi rx is above the
+		 * high water mark
+		 */
+		lws_rx_flow_control(m->ss->wsi, 0);
+		m->conn->onward_in_flow_control = 1;
+	}
 
 	if (m->conn->wsi) /* if possible, request client conn write */
 		lws_callback_on_writable(m->conn->wsi);
 
-	return 0;
+	return LWSSSSRET_OK;
 }
 
 /*
@@ -128,23 +192,26 @@ ss_proxy_onward_tx(void *userobj, lws_ss_tx_ordinal_t ord, uint8_t *buf,
 		lwsl_notice("%s: ss not ready\n", __func__);
 		*len = 0;
 
-		return 1;
+		return LWSSSSRET_TX_DONT_SEND;
 	}
 
 	/*
 	 * The onward secure stream says that we could send something to it
-	 * (by putting it in buf, and setting *len and *flags)
+	 * (by putting it in buf, and setting *len and *flags)... dredge the
+	 * next thing out of the dsh
 	 */
 
 	if (lws_ss_deserialize_tx_payload(m->conn->dsh, m->ss->wsi,
 					  ord, buf, len, flags))
-		return 1;
+		return LWSSSSRET_TX_DONT_SEND;
 
+	/* ... there's more we want to send? */
 	if (!lws_dsh_get_head(m->conn->dsh, KIND_C_TO_P, (void **)&p, &si))
-		lws_ss_request_tx(m->conn->ss);
+		_lws_ss_request_tx(m->conn->ss);
 
 	if (!*len && !*flags)
-		return 1; /* we don't actually want to send anything */
+		/* we don't actually want to send anything */
+		return LWSSSSRET_TX_DONT_SEND;
 
 	lwsl_info("%s: onward tx %d fl 0x%x\n", __func__, (int)*len, *flags);
 
@@ -158,7 +225,7 @@ ss_proxy_onward_tx(void *userobj, lws_ss_tx_ordinal_t ord, uint8_t *buf,
 	}
 #endif
 
-	return 0;
+	return LWSSSSRET_OK;
 }
 
 static lws_ss_state_return_t
@@ -166,9 +233,36 @@ ss_proxy_onward_state(void *userobj, void *sh,
 		      lws_ss_constate_t state, lws_ss_tx_ordinal_t ack)
 {
 	ss_proxy_t *m = (ss_proxy_t *)userobj;
+	size_t dsh_size;
 
 	switch (state) {
 	case LWSSSCS_CREATING:
+
+		/*
+		 * conn is private to -process.c, call thru to a) adjust
+		 * the accepted incoming proxy link wsi tag name to be
+		 * appended with the onward ss tag information now we
+		 * have it, and b) allocate the dsh buffer now we
+		 * can find out the policy about it for the streamtype.
+		 */
+
+		dsh_size = m->ss->policy->proxy_buflen ?
+				m->ss->policy->proxy_buflen : 32768;
+
+		lwsl_notice("%s: %s: initializing dsh max len %lu\n",
+				__func__, lws_ss_tag(m->ss),
+				(unsigned long)dsh_size);
+
+		/* this includes ssproxy_dsh_create_oom fault generation */
+
+		if (__lws_ss_proxy_bind_ss_to_conn_wsi(m->conn, dsh_size)) {
+
+			/* failed to allocate the dsh */
+
+			lwsl_notice("%s: dsh init failed\n", __func__);
+
+			return LWSSSSRET_DESTROY_ME;
+		}
 		break;
 
 	case LWSSSCS_DESTROYING:
@@ -194,15 +288,20 @@ ss_proxy_onward_state(void *userobj, void *sh,
 	if (!m->conn) {
 		lwsl_warn("%s: dropping state due to conn not up\n", __func__);
 
-		return 0;
+		return LWSSSSRET_OK;
 	}
 
-	lws_ss_serialize_state(m->conn->dsh, state, ack);
+	if (lws_ss_serialize_state(m->conn->wsi, m->conn->dsh, state, ack))
+		/*
+		 * Failed to alloc state packet that we want to send in dsh,
+		 * we will lose coherence and have to disconnect the link
+		 */
+		return LWSSSSRET_DISCONNECT_ME;
 
 	if (m->conn->wsi) /* if possible, request client conn write */
 		lws_callback_on_writable(m->conn->wsi);
 
-	return 0;
+	return LWSSSSRET_OK;
 }
 
 void
@@ -220,23 +319,20 @@ ss_proxy_onward_txcr(void *userobj, int bump)
 }
 
 /*
- * Client - Proxy connection on unix domain socket
+ * Client <-> Proxy connection, usually on Unix Domain Socket
  */
 
 static int
 callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		  void *user, void *in, size_t len)
 {
-	struct lws_context_per_thread *pt = &wsi->a.context->pt[(int)wsi->tsi];
 	struct raw_pss *pss = (struct raw_pss *)user;
 	const lws_ss_policy_t *rsp;
 	struct conn *conn = NULL;
+	lws_ss_metadata_t *md;
 	lws_ss_info_t ssi;
 	const uint8_t *cp;
-#if defined(LWS_WITH_DETAILED_LATENCY)
-	lws_usec_t us;
-#endif
-	char s[128];
+	char s[512];
 	uint8_t *p;
 	size_t si;
 	char pay;
@@ -258,20 +354,21 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		lwsl_info("LWS_CALLBACK_RAW_ADOPT\n");
 		if (!pss)
 			return -1;
-		pss->conn = malloc(sizeof(struct conn));
+
+		if (lws_fi(&wsi->fic, "ssproxy_client_adopt_oom"))
+			pss->conn = NULL;
+		else
+			pss->conn = malloc(sizeof(struct conn));
 		if (!pss->conn)
 			return -1;
+
 		memset(pss->conn, 0, sizeof(*pss->conn));
 
-		pss->conn->dsh = lws_dsh_create(&pt->ss_dsh_owner,
-						LWS_SS_MTU * 160, 2);
-		if (!pss->conn->dsh) {
-			free(pss->conn);
-
-			return -1;
-		}
+		/* dsh is allocated when the onward ss is done */
 
 		pss->conn->wsi = wsi;
+		wsi->bound_ss_proxy_conn = 1; /* opaque is conn */
+
 		pss->conn->state = LPCSPROX_WAIT_INITIAL_TX;
 
 		/*
@@ -279,8 +376,7 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		 * acceptance up rapidly with an initial tx containing the
 		 * streamtype name.  We can't create the stream until then.
 		 */
-		lws_set_timeout(wsi,
-				PENDING_TIMEOUT_AWAITING_CLIENT_HS_SEND, 3);
+		lws_set_timeout(wsi, PENDING_TIMEOUT_AWAITING_CLIENT_HS_SEND, 3);
                 break;
 
 	case LWS_CALLBACK_RAW_CLOSE:
@@ -298,16 +394,43 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		 * still live...
 		 */
 
+		assert(conn->wsi == wsi);
+		conn->wsi = NULL;
+
+		lwsl_notice("%s: cli->prox link %s closing\n", __func__,
+				lws_wsi_tag(wsi));
+
+		/* sever relationship with conn */
+		lws_set_opaque_user_data(wsi, NULL);
+
+		/*
+		 * The current wsi is decoupled from the pss / conn and
+		 * the conn no longer has a pointer on it.
+		 *
+		 * If there's an outgoing, proxied SS conn on our behalf, we
+		 * have to destroy those
+		 */
+
 		if (conn->ss) {
 			struct lws *cw = conn->ss->wsi;
 			/*
-			 * The onward connection is around
+			 * conn->ss is the onward connection SS
 			 */
-			lwsl_info("%s: destroying ss.h=%p, ss.wsi=%p\n",
-					__func__, conn->ss, conn->ss->wsi);
-			/* sever relationship with ss about to be deleted */
-			lws_set_opaque_user_data(wsi, NULL);
-			if (wsi != cw)
+
+			lwsl_info("%s: destroying %s, wsi %s\n",
+					__func__, lws_ss_tag(conn->ss),
+					lws_wsi_tag(conn->ss->wsi));
+
+			/* sever conn relationship with ss about to be deleted */
+
+			conn->ss->wsi = NULL;
+
+			if (cw && wsi != cw) {
+
+				/* disconnect onward SS from its wsi */
+
+				lws_set_opaque_user_data(cw, NULL);
+
 				/*
 				 * The wsi doing the onward connection can no
 				 * longer relate to the conn... otherwise when
@@ -315,11 +438,13 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 				 * the ss we are about to delete
 				 */
 				lws_wsi_close(cw, LWS_TO_KILL_ASYNC);
-			conn->wsi = NULL;
-
+			}
 
 			lws_ss_destroy(&conn->ss);
-			/* conn may have gone */
+			/*
+			 * Conn may have gone, at ss destroy handler in
+			 * ssi.state for proxied ss
+			 */
 			break;
 		}
 
@@ -332,7 +457,7 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 			free(conn);
 			pss->conn = NULL;
 		} else
-			lwsl_debug("%s: CLOSE; ss=%p\n", __func__, conn->ss);
+			lwsl_debug("%s: CLOSE; %s\n", __func__, lws_ss_tag(conn->ss));
 
 		break;
 
@@ -362,6 +487,7 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		ssi.state = ss_proxy_onward_state;
 		ssi.flags = 0;
 
+		// coverity[uninit_use_in_call]
 		n = lws_ss_deserialize_parse(&conn->parser,
 				lws_get_context(wsi), conn->dsh, in, len,
 				&conn->state, conn, &conn->ss, &ssi, 0);
@@ -383,7 +509,9 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 		break;
 
 	case LWS_CALLBACK_RAW_WRITEABLE:
-		// lwsl_notice("LWS_CALLBACK_RAW_PROXY_SRV_WRITEABLE\n");
+
+		lwsl_debug("%s: %s: LWS_CALLBACK_RAW_WRITEABLE, state 0x%x\n",
+				__func__, lws_wsi_tag(wsi), lwsi_state(wsi));
 
 		/*
 		 * We can transmit something back to the client from the dsh
@@ -395,6 +523,7 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 
 		n = 0;
 		pay = 0;
+
 		s[3] = 0;
 		cp = (const uint8_t *)s;
 		switch (conn->state) {
@@ -406,7 +535,11 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 			s[1] = 0;
 			s[2] = 1;
 
-			n = 4;
+			n = 8;
+
+			lws_ser_wu32be((uint8_t *)&s[4], conn->ss &&
+							 conn->ss->policy ?
+					conn->ss->policy->client_buflen : 0);
 
 			/*
 			 * If there's rideshare sequencing, it's added after the
@@ -419,23 +552,94 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 				while (rsp) {
 					if (n != 4 && n < (int)sizeof(s) - 2)
 						s[n++] = ',';
-					n += lws_snprintf(&s[n], sizeof(s) - n,
+					n += lws_snprintf(&s[n], sizeof(s) - (unsigned int)n,
 							"%s", rsp->streamtype);
 					rsp = lws_ss_policy_lookup(wsi->a.context,
 						rsp->rideshare_streamtype);
 				}
 			}
-			s[2] = n - 3;
+			s[2] = (char)(n - 3);
 			conn->state = LPCSPROX_OPERATIONAL;
 			lws_set_timeout(wsi, 0, 0);
 			break;
+
 		case LPCSPROX_OPERATIONAL:
+
+			/*
+			 * returning [onward -> ] proxy]-> client
+			 * rx metadata has priority 1
+			 */
+
+			md = conn->ss->metadata;
+			while (md) {
+				// lwsl_notice("%s: check %s: %d\n", __func__,
+				// md->name, md->pending_onward);
+				if (md->pending_onward) {
+					size_t naml = strlen(md->name);
+
+					// lwsl_notice("%s: proxy issuing rxmd\n", __func__);
+
+					if (4 + naml + md->length > sizeof(s)) {
+						lwsl_err("%s: rxmdata too big\n",
+								__func__);
+						goto hangup;
+					}
+					md->pending_onward = 0;
+					p = (uint8_t *)s;
+					p[0] = LWSSS_SER_RXPRE_METADATA;
+					lws_ser_wu16be(&p[1], (uint16_t)(1 + naml +
+							      md->length));
+					p[3] = (uint8_t)naml;
+					memcpy(&p[4], md->name, naml);
+					p += 4 + naml;
+					memcpy(p, md->value__may_own_heap,
+					       md->length);
+					p += md->length;
+
+					n = lws_ptr_diff(p, cp);
+					goto again;
+				}
+
+				md = md->next;
+			}
+
+			/*
+			 * If we have performance data, render it in JSON
+			 * and send that in LWSSS_SER_RXPRE_PERF has
+			 * priority 2
+			 */
+
+#if defined(LWS_WITH_CONMON)
+			if (conn->ss->conmon_json) {
+				unsigned int xlen = conn->ss->conmon_len;
+
+				if (xlen > sizeof(s) - 3)
+					xlen = sizeof(s) - 3;
+				cp = (uint8_t *)s;
+				p = (uint8_t *)s;
+				p[0] = LWSSS_SER_RXPRE_PERF;
+				lws_ser_wu16be(&p[1], (uint16_t)xlen);
+				memcpy(&p[3], conn->ss->conmon_json, xlen);
+
+				lws_free_set_NULL(conn->ss->conmon_json);
+				n = (int)(xlen + 3);
+
+				pay = 0;
+				goto again;
+			}
+#endif
+			/*
+			 * if no fresh rx metadata, just pass through incoming
+			 * dsh
+			 */
+
 			if (lws_dsh_get_head(conn->dsh, KIND_SS_TO_P,
 					     (void **)&p, &si))
 				break;
+
 			cp = p;
 
-#if defined(LWS_WITH_DETAILED_LATENCY)
+#if 0
 			if (cp[0] == LWSSS_SER_RXPRE_RX_PAYLOAD &&
 			    wsi->a.context->detailed_latency_cb) {
 
@@ -464,7 +668,6 @@ callback_ss_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 							lws_ser_ru32be(&p[7]);
 			}
 #endif
-
 			pay = 1;
 			n = (int)si;
 			break;
@@ -475,7 +678,10 @@ again:
 		if (!n)
 			break;
 
-		n = lws_write(wsi, (uint8_t *)cp, n, LWS_WRITE_RAW);
+		if (lws_fi(&wsi->fic, "ssproxy_client_write_fail"))
+			n = -1;
+		else
+			n = lws_write(wsi, (uint8_t *)cp, (unsigned int)n, LWS_WRITE_RAW);
 		if (n < 0) {
 			lwsl_info("%s: WRITEABLE: %d\n", __func__, n);
 
@@ -486,8 +692,35 @@ again:
 		case LPCSPROX_REPORTING_FAIL:
 			goto hangup;
 		case LPCSPROX_OPERATIONAL:
-			if (pay)
+			if (!conn)
+				break;
+			if (pay) {
 				lws_dsh_free((void **)&p);
+
+				/*
+				 * Did we go below the rx flow threshold for
+				 * this dsh?
+				 */
+
+				if (conn->onward_in_flow_control &&
+				    conn->ss->policy->proxy_buflen_rxflow_on_above &&
+				    conn->ss->wsi &&
+				    lws_dsh_get_size(conn->dsh, KIND_SS_TO_P) <
+				      conn->ss->policy->proxy_buflen_rxflow_off_below) {
+					lwsl_info("%s: %s: rxflow enabling rx (%lu / %lu, lwm %lu)\n", __func__,
+							lws_wsi_tag(conn->ss->wsi),
+							(unsigned long)lws_dsh_get_size(conn->dsh, KIND_SS_TO_P),
+							(unsigned long)conn->ss->policy->proxy_buflen,
+							(unsigned long)conn->ss->policy->proxy_buflen_rxflow_off_below);
+					/*
+					 * Resume receiving taking in rx once
+					 * below the low threshold
+					 */
+					lws_rx_flow_control(conn->ss->wsi,
+							    LWS_RXFLOW_ALLOW);
+					conn->onward_in_flow_control = 0;
+				}
+			}
 			if (!lws_dsh_get_head(conn->dsh, KIND_SS_TO_P,
 					     (void **)&p, &si)) {
 				if (!lws_send_pipe_choked(wsi)) {
@@ -511,10 +744,8 @@ again:
 	return lws_callback_http_dummy(wsi, reason, user, in, len);
 
 hangup:
-	//lws_ss_destroy(&conn->ss);
-	//conn->state = LPCSPROX_DESTROYED;
-
 	/* hang up on him */
+
 	return -1;
 }
 
@@ -540,15 +771,23 @@ lws_ss_proxy_create(struct lws_context *context, const char *bind, int port)
 	memset(&info, 0, sizeof(info));
 
 	info.vhost_name			= "ssproxy";
-	info.options = LWS_SERVER_OPTION_ADOPT_APPLY_LISTEN_ACCEPT_CONFIG;
+	info.options = LWS_SERVER_OPTION_ADOPT_APPLY_LISTEN_ACCEPT_CONFIG |
+			LWS_SERVER_OPTION_SS_PROXY;
 	info.port = port;
 	if (!port) {
 		if (!bind)
+#if defined(__linux__)
 			bind = "@proxy.ss.lws";
+#else
+			bind = "/tmp/proxy.ss.lws";
+#endif
 		info.options |= LWS_SERVER_OPTION_UNIX_SOCK;
 	}
 	info.iface			= bind;
+#if defined(__linux__)
 	info.unix_socket_perms		= "root:root";
+#else
+#endif
 	info.listen_accept_role		= "raw-skt";
 	info.listen_accept_protocol	= "ssproxy-protocol";
 	info.protocols			= protocols;
