@@ -151,14 +151,20 @@ static const char * const default_ss_policy =
 			"]"
 		"}"
 	  "],"
+	  "\"auth\": [" /* available auth type bindings */
+		"{"
+		  "\"name\":"		"\"lwa\","
+		  "\"streamtype\":"	"\"api_amazon_com_lwa\","
+		  "\"blob\":"		"0"
+		"}"
+	  "],"
 	  "\"s\": [" /* the supported stream types */
-		"{\"api_amazon_com_auth\": {"
+		"{\"api_amazon_com_lwa\": {"
 			"\"endpoint\":"			"\"api.amazon.com\","
 			"\"port\":"			"443,"
 			"\"protocol\":"			"\"h1\","
 			"\"http_method\":"		"\"POST\","
 			"\"http_url\":"			"\"auth/o2/token\","
-			"\"plugins\":"			"[],"
 			"\"opportunistic\":"		"true,"
 			"\"tls\":"			"true,"
 			"\"h2q_oflow_txcr\":"		"true,"
@@ -176,10 +182,10 @@ static const char * const default_ss_policy =
 			"\"h2q_oflow_txcr\":"		"true,"
 			"\"http_auth_header\":"		"\"authorization:\","
 			"\"http_auth_preamble\":"	"\"Bearer \","
+			"\"use_auth\":"			"\"lwa\","
 			"\"nailed_up\":"		"true,"
 			"\"long_poll\":"		"true,"
 			"\"retry\":"			"\"default\","
-			"\"plugins\":"			"[],"
 			"\"tls\":"			"true,"
 			"\"tls_trust_store\":"		"\"avs_via_starfield\""
 		"}},"
@@ -191,6 +197,7 @@ static const char * const default_ss_policy =
 			"\"http_url\":"			"\"v20160207/events\","
 			"\"http_no_content_length\":"	"true,"
 			"\"h2q_oflow_txcr\":"		"true,"
+			"\"use_auth\":"			"\"lwa\","
 			"\"http_auth_header\":"		"\"authorization:\","
 			"\"http_auth_preamble\":"	"\"Bearer \","
 			"\"http_multipart_name\":"	"\"metadata\","
@@ -200,7 +207,6 @@ static const char * const default_ss_policy =
 #endif
 			"\"rideshare\":"		"\"avs_audio\","
 			"\"retry\":"			"\"default\","
-			"\"plugins\":"			"[],"
 			"\"tls\":"			"true,"
 			"\"tls_trust_store\":"		"\"avs_via_starfield\""
 		"}},"
@@ -211,12 +217,12 @@ static const char * const default_ss_policy =
 			"\"http_method\":"		"\"POST\","
 			"\"http_url\":"			"\"v20160207/events\","
 			"\"http_no_content_length\":"	"true,"
-			"\"plugins\":"			"[],"
 			"\"tls\":"			"true,"
 			"\"h2q_oflow_txcr\":"		"true,"
 #if 1
 			"\"http_multipart_ss_in\":"	"true,"
 #endif
+			"\"use_auth\":"			"\"lwa\","
 			"\"http_auth_header\":"		"\"authorization:\","
 			"\"http_auth_preamble\":"	"\"Bearer \","
 			"\"http_multipart_name\":"	"\"audio\","
@@ -335,16 +341,13 @@ int main(int argc, const char **argv)
 	}
 #endif
 
-#if defined(LWS_WITH_DETAILED_LATENCY)
-	info.detailed_latency_cb = lws_det_lat_plot_cb;
-	info.detailed_latency_filepath = "/tmp/lws-latency-ssproxy";
-#endif
-
 	/* integrate us with lws system state management when context created */
 	nl.name = "app";
 	nl.notify_cb = app_system_state_nf;
 	info.register_notifier_list = app_notifier_list;
 
+	puts(default_ss_policy);
+
 	context = lws_create_context(&info);
 	if (!context) {
 		lwsl_err("lws init failed\n");
