@@ -1,7 +1,7 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2020 Andy Green <andy@warmcat.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -23,8 +23,12 @@
  */
 
 #if !defined (LWS_PLUGIN_STATIC)
+#if !defined(LWS_DLL)
 #define LWS_DLL
+#endif
+#if !defined(LWS_INTERNAL)
 #define LWS_INTERNAL
+#endif
 #include <libwebsockets.h>
 #endif
 
@@ -129,11 +133,12 @@ static int
 scan_upload_dir(struct vhd_deaddrop *vhd)
 {
 	char filepath[256], subdir[3][128], *p;
-	int m, sp = 0, initial, found = 0;
 	struct lwsac *lwsac_head = NULL;
 	lws_list_ptr sorted_head = NULL;
+	int i, sp = 0, found = 0;
 	struct dir_entry *dire;
 	struct dirent *de;
+	size_t initial, m;
 	struct stat s;
 	DIR *dir[3];
 
@@ -162,11 +167,11 @@ scan_upload_dir(struct vhd_deaddrop *vhd)
 
 		p = filepath;
 
-		for (m = 0; m <= sp; m++)
-			p += lws_snprintf(p, (filepath + sizeof(filepath)) - p,
-					  "%s/", subdir[m]);
+		for (i = 0; i <= sp; i++)
+			p += lws_snprintf(p, lws_ptr_diff_size_t((filepath + sizeof(filepath)), p),
+					  "%s/", subdir[i]);
 
-		lws_snprintf(p, (filepath + sizeof(filepath)) - p, "%s",
+		lws_snprintf(p, lws_ptr_diff_size_t((filepath + sizeof(filepath)), p), "%s",
 				  de->d_name);
 
 		/* ignore temp files */
@@ -211,7 +216,7 @@ scan_upload_dir(struct vhd_deaddrop *vhd)
 		}
 
 		dire->next = NULL;
-		dire->size = s.st_size;
+		dire->size = (unsigned long long)s.st_size;
 		dire->mtime = s.st_mtime;
 		dire->user[0] = '\0';
 #if !defined(__COVERITY__)
@@ -257,10 +262,11 @@ bail:
 
 static int
 file_upload_cb(void *data, const char *name, const char *filename,
-	       char *buf, int len, enum lws_spa_fileupload_states state)
+	       char *buf, int _len, enum lws_spa_fileupload_states state)
 {
 	struct pss_deaddrop *pss = (struct pss_deaddrop *)data;
 	char filename2[256];
+	size_t len = (size_t)_len;
 	int n;
 
 	(void)n;
@@ -300,7 +306,7 @@ file_upload_cb(void *data, const char *name, const char *filename,
 	case LWS_UFS_FINAL_CONTENT:
 	case LWS_UFS_CONTENT:
 		if (len) {
-			pss->file_length += len;
+			pss->file_length += (unsigned int)len;
 
 			/* if the file length is too big, drop it */
 			if (pss->file_length > pss->vhd->max_size) {
@@ -314,9 +320,9 @@ file_upload_cb(void *data, const char *name, const char *filename,
 			}
 
 			if (pss->fd != LWS_INVALID_FILE) {
-				n = write((int)(lws_intptr_t)pss->fd, buf, len);
+				n = (int)write((int)(lws_intptr_t)pss->fd, buf, (unsigned int)len);
 				lwsl_debug("%s: write %d says %d\n", __func__,
-					   len, n);
+					   (int)len, n);
 				lws_set_timeout(pss->wsi, PENDING_TIMEOUT_HTTP_CONTENT, 30);
 			}
 		}
@@ -357,12 +363,12 @@ format_result(struct pss_deaddrop *pss)
 	start = p;
 	end = p + sizeof(pss->result) - LWS_PRE - 1;
 
-	p += lws_snprintf((char *)p, end -p,
+	p += lws_snprintf((char *)p, lws_ptr_diff_size_t(end, p),
 			"<!DOCTYPE html><html lang=\"en\"><head>"
 			"<meta charset=utf-8 http-equiv=\"Content-Language\" "
 			"content=\"en\"/>"
 			"</head>");
-	p += lws_snprintf((char *)p, end - p, "</body></html>");
+	p += lws_snprintf((char *)p, lws_ptr_diff_size_t(end, p), "</body></html>");
 
 	return (int)lws_ptr_diff(p, start);
 }
@@ -377,7 +383,7 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 	struct pss_deaddrop *pss = (struct pss_deaddrop *)user;
 	uint8_t buf[LWS_PRE + LWS_RECOMMENDED_MIN_HEADER_SPACE],
 		*start = &buf[LWS_PRE], *p = start,
-		*end = &buf[sizeof(buf) - LWS_PRE - 1];
+		*end = &buf[sizeof(buf) - 1];
 	char fname[256], *wp;
 	const char *cp;
 	int n, m, was;
@@ -392,6 +398,8 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 		vhd = (struct vhd_deaddrop *)
 			lws_protocol_vh_priv_get(lws_get_vhost(wsi),
 						 lws_get_protocol(wsi));
+		if (!vhd)
+			return 0;
 
 		vhd->context = lws_get_context(wsi);
 		vhd->vh = lws_get_vhost(wsi);
@@ -399,10 +407,10 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 		vhd->max_size = 20 * 1024 * 1024; /* default without pvo */
 
 		if (!lws_pvo_get_str(in, "max-size", &cp))
-			vhd->max_size = atoll(cp);
+			vhd->max_size = (unsigned long long)atoll(cp);
 		if (lws_pvo_get_str(in, "upload-dir", &vhd->upload_dir)) {
-			lwsl_err("%s: requires 'upload-dir' pvo\n", __func__);
-			return -1;
+			lwsl_warn("%s: requires 'upload-dir' pvo\n", __func__);
+			return 0;
 		}
 
 		scan_upload_dir(vhd);
@@ -413,7 +421,8 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 		break;
 
 	case LWS_CALLBACK_PROTOCOL_DESTROY:
-		lwsac_free(&vhd->lwsac_head);
+		if (vhd)
+			lwsac_free(&vhd->lwsac_head);
 		break;
 
 	/* WS-related */
@@ -460,10 +469,10 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 
 		cp = strchr((const char *)in, '/');
 		if (cp) {
-			n = ((void *)cp - in) - 8;
+			n = (int)(((void *)cp - in)) - 8;
 
 			if ((int)strlen(pss->user) != n ||
-			    memcmp(pss->user, ((const char *)in) + 8, n)) {
+			    memcmp(pss->user, ((const char *)in) + 8, (unsigned int)n)) {
 				lwsl_notice("%s: del: auth mismatch "
 					    " '%s' '%s' (%d)\n",
 					    __func__, pss->user,
@@ -496,7 +505,7 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 
 		was = 0;
 		if (pss->first) {
-			p += lws_snprintf((char *)p, lws_ptr_diff(end, p),
+			p += lws_snprintf((char *)p, lws_ptr_diff_size_t(end, p),
 					  "{\"max_size\":%llu, \"files\": [",
 					  vhd->max_size);
 			was = 1;
@@ -504,7 +513,7 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 
 		m = 5;
 		while (m-- && pss->dire) {
-			p += lws_snprintf((char *)p, lws_ptr_diff(end, p),
+			p += lws_snprintf((char *)p, lws_ptr_diff_size_t(end, p),
 					  "%c{\"name\":\"%s\", "
 					  "\"size\":%llu,"
 					  "\"mtime\":%llu,"
@@ -520,7 +529,7 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 		}
 
 		if (!pss->dire) {
-			p += lws_snprintf((char *)p, lws_ptr_diff(end, p),
+			p += lws_snprintf((char *)p, lws_ptr_diff_size_t(end, p),
 					  "]}");
 			if (pss->lwsac_head) {
 				lwsac_unreference(&pss->lwsac_head);
@@ -528,8 +537,8 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 			}
 		}
 
-		n = lws_write(wsi, start, lws_ptr_diff(p, start),
-			      lws_write_ws_flags(LWS_WRITE_TEXT, was,
+		n = lws_write(wsi, start, lws_ptr_diff_size_t(p, start),
+				(enum lws_write_protocol)lws_write_ws_flags(LWS_WRITE_TEXT, was,
 						 !pss->dire));
 		if (n < 0) {
 			lwsl_notice("%s: ws write failed\n", __func__);
@@ -609,7 +618,8 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 		if (!pss->sent_headers) {
 			n = format_result(pss);
 
-			if (lws_add_http_header_status(wsi, pss->response_code,
+			if (lws_add_http_header_status(wsi,
+					(unsigned int)pss->response_code,
 						       &p, end))
 				goto bail;
 
@@ -618,13 +628,13 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 					(unsigned char *)"text/html", 9,
 					&p, end))
 				goto bail;
-			if (lws_add_http_header_content_length(wsi, n, &p, end))
+			if (lws_add_http_header_content_length(wsi, (lws_filepos_t)n, &p, end))
 				goto bail;
 			if (lws_finalize_http_header(wsi, &p, end))
 				goto bail;
 
 			/* first send the headers ... */
-			n = lws_write(wsi, start, lws_ptr_diff(p, start),
+			n = lws_write(wsi, start, lws_ptr_diff_size_t(p, start),
 				      LWS_WRITE_HTTP_HEADERS |
 				      LWS_WRITE_H2_STREAM_END);
 			if (n < 0)
@@ -637,7 +647,7 @@ callback_deaddrop(struct lws *wsi, enum lws_callback_reasons reason,
 
 		if (!pss->sent_body) {
 			n = format_result(pss);
-			n = lws_write(wsi, (unsigned char *)start, n,
+			n = lws_write(wsi, (unsigned char *)start, (unsigned int)n,
 				      LWS_WRITE_HTTP_FINAL);
 
 			pss->sent_body = 1;
@@ -685,7 +695,7 @@ try_to_reuse:
 
 #if !defined (LWS_PLUGIN_STATIC)
 
-static const struct lws_protocols protocols[] = {
+LWS_VISIBLE const struct lws_protocols deaddrop_protocols[] = {
 	LWS_PLUGIN_PROTOCOL_DEADDROP
 };
 
@@ -693,11 +703,12 @@ LWS_VISIBLE const lws_plugin_protocol_t deaddrop = {
 	.hdr = {
 		"deaddrop",
 		"lws_protocol_plugin",
+		LWS_BUILD_HASH,
 		LWS_PLUGIN_API_MAGIC
 	},
 
-	.protocols = protocols,
-	.count_protocols = LWS_ARRAY_SIZE(protocols),
+	.protocols = deaddrop_protocols,
+	.count_protocols = LWS_ARRAY_SIZE(deaddrop_protocols),
 	.extensions = NULL,
 	.count_extensions = 0,
 };
